// 04.09.2018 - WWPumpenzeit - If-Anweisung zur Zeitabfrage neu gebaut. 
// 05.09.2018 - Kesselzeiten - If-Anweisung von WWPumenzeit nach Test übernommen
// 06.09.2018 - Heizungsregelungssoftware in Git Respo
#include <OneWire.h> // http://www.arduino.cc/playground/Learning/OneWire
#include <DallasTemperature.h> // http://milesburton.com/index.php?title=Dallas_Temperature_Control_Library
#include <Wire.h>
#include "RTClib.h"

#define ONE_WIRE_BUS 2
#define TEMPERATURE_PRECISION 12
OneWire oneWire(ONE_WIRE_BUS); // Einrichten des OneWire Bus um die Daten der Temperaturfühler abzurufen
DallasTemperature sensors(&oneWire); // Bindung der Sensoren an den OneWire Bus
DeviceAddress tempDeviceAddress; // Verzeichniss zum Speichern von Sensor Adressen

const int DS18S20_Pin = 2; //DS18S20 Signal pin on digital 2
const int KesselPoti = A0;  // Eingestellte Kesseltemp
const int HysterresePoti = A1; // Hysterrese der Kesseltemp
const int BrennerRelais = 10;
const int WW_PumpRelais = 11;
const int HeizPumpRelais = 12;
// Termosensoren Nummern
const int SensorKesseltemp = 0; // Sensor-Nummer Kesseltemperatur
const int SensorAussentemp = 0; // Sensor-Nummer Kesseltemperatur

bool Nacht = false; // Nachtabsenkung aktiv
bool EinFlag = false; // Flags für Brennerzeiten
bool AusFlag = false;
bool WWEinFlag = false; // Flags für WW Pumpe
bool WWAusFlag = false;
int ZeitCnt;
int WWZeitCnt;
int numberOfDevices; // Anzahl der gefundenen Temperatur Sensoren


// Temperaturen
float Absenkung = 10;  // Um diese Temperatur wird die KesselTemp verringert
float HysterreseTemp = 6; // Festgelegte Hysterrese
float AussentempEin = 16; // Unterhalb dieser Temp. wird die Raumheizung eingeschaltet
float AussentempHyst = 4;  // Hysterrese für die Aussentemperatur (wird zur AussentempEin zuadiert!)


// Kesselzeiten
int Kesselzeit[15][5] = { // {Tag, Startzeit, EndZeit}
  {99, 99, 99},
  {1, 545, 700}, // Montag
  {1, 1600, 2100},
  {2, 545, 700}, // Dienstag
  {2, 1600, 2100},
  {3, 545, 700}, // Mittwoch
  {3, 1600, 2100},
  {4, 545, 700}, // Donnerstag
  {4, 1600, 2100},
  {5, 545, 700}, // Freitag
  {5, 1200, 2100},
  {6, 545, 2200}, // Samstag
  //{6,16,0,21,0},
  {0, 545, 2100}, // Sonntag
  //{7,16,0,21,0}
};
int KesselzeitCnt=13; // Anzahl der Zeiten im Array

// Umwälspumpe Zeiten
int WWPumpe[9][4] = { // {Tag, Startzeit, EndZeit}
  {9999, 9999, 9999},
  {1, 630, 700}, // Montag
  {2, 630, 700}, // Dienstag
  {3, 630, 700}, // Mittwoch
  {4, 630, 700}, // Donnerstag
  {5, 630, 700}, // Freitag
  {6, 630, 900}, // Samstag
  {0, 630, 900} // Sonntag 
  //{0, 1232, 1233} //Debug Zeit
};
int WWPumpeCnt=8; // Anzahl der Einträge im Array

// Heizkörper Zeiten
RTC_DS1307 RTC;
DateTime now; 
OneWire ds(DS18S20_Pin);  // on digital pin 2

void setup() {
  Serial.begin(9600);

  //Init Temperatursensoren
   numberOfDevices = sensors.getDeviceCount();
   Serial.println("Abfrage mehrerer Dallas Temperatur Sensoren");
   Serial.println("-------------------------------------------");
 
   // Suche der Sensoren
   Serial.println("Suche Temperatur Sensoren...");
   sensors.begin();
   numberOfDevices = sensors.getDeviceCount();
 
   Serial.print("Habe ");
   Serial.print(numberOfDevices, DEC);
   Serial.println(" Sensoren gefunden.");
 
   // Setzen der Genauigkeit
   for(int i=0 ;i<numberOfDevices; i++) {
      if(sensors.getAddress(tempDeviceAddress, i)) {
         sensors.setResolution(tempDeviceAddress, TEMPERATURE_PRECISION);
         Serial.print("Sensor ");
         Serial.print(i);
         Serial.print(" hat eine genauigkeit von ");
         Serial.println(sensors.getResolution(tempDeviceAddress), DEC);
      }
   }
   Serial.println("");

   
  // Init RTC
  Wire.begin();
  RTC.begin();
  if (! RTC.isrunning()) {
    Serial.println("RTC is NOT running!");
    // following line sets the RTC to the date & time this sketch was compiled
    //RTC.adjust(DateTime((__DATE__), (__TIME__)));
  }
  
  // initialize the Relais pin as an output:
  pinMode(BrennerRelais, OUTPUT);
  pinMode(WW_PumpRelais, OUTPUT);
  pinMode(HeizPumpRelais, OUTPUT);

  // Methoden deklarieren
  void relaistest();
  int analogtest();
  void zeittest();

  void Kessel(); // Kesseltemperatur regeln
  bool TagNachtZeit();
  void WWPumpenZeit();
  void Aussentemperatur();
  void Zeit();

  digitalWrite(BrennerRelais, LOW);
  digitalWrite(WW_PumpRelais, LOW);
  digitalWrite(HeizPumpRelais, LOW);
} // ######Setup Ende######


void loop() {
  now = RTC.now(); 
  Zeit();
  Nacht = TagNachtZeit();
  Kesseltemperatur(Nacht);
  WWPumpenZeit();
  Aussentemperatur();
} // ##### Main-Loop Ende ####


// ############# Erst klären, wie die Adressierung von den Temperatursensoren funktioniert!!!! ##############
void Aussentemperatur(){
  

  float AussentempAnalog = getTemp(SensorAussentemp); // Aktuelle Kesseltemp. holen ----> Adresse von Fühler
  if (AussentempAnalog <= AussentempEin){
    digitalWrite(HeizPumpRelais, HIGH);
  }else if (AussentempAnalog >= (AussentempEin + AussentempHyst)){
    digitalWrite(HeizPumpRelais, LOW);
  }
}


void Zeit(){
    now = RTC.now();
    Serial.println();
    Serial.print(now.dayOfWeek(), DEC);
    Serial.println(); 
    Serial.print(now.year(), DEC);
    Serial.print('/');
    Serial.print(now.month(), DEC);
    Serial.print('/');
    Serial.print(now.day(), DEC);
    Serial.print(' ');
    Serial.print(now.hour(), DEC);
    Serial.print(':');
    Serial.print(now.minute(), DEC);
    Serial.print(':');
    Serial.print(now.second(), DEC);
    Serial.println();
}


void WWPumpenZeit(){
  now = RTC.now();
  int TagJetzt = (int) now.dayOfWeek();
  int MinuteJetzt = (int) now.minute();
  int StundeJetzt = (int) now.hour(); 
  int ZeitJetzt=StundeJetzt*100+MinuteJetzt;

  for (int cnt=0; cnt<WWPumpeCnt; cnt++){
    int ZeitStartProgramm=WWPumpe[cnt][1];
    int ZeitEndeProgramm=WWPumpe[cnt][2];
    if(WWPumpe[cnt][0]==TagJetzt){ // Tag aus Array 
      if(ZeitJetzt == ZeitStartProgramm){ // Stunde liegt zwischen
        digitalWrite(WW_PumpRelais, HIGH);
      } else if(ZeitJetzt == ZeitEndeProgramm){
        digitalWrite(WW_PumpRelais, LOW);
      }
    }
  }
}


// Kesselzeiten
bool TagNachtZeit(){
  now = RTC.now();
  int TagJetzt = (int) now.dayOfWeek();
  int MinuteJetzt = (int) now.minute();
  int StundeJetzt = (int) now.hour(); 
  int ZeitJetzt=StundeJetzt*100+MinuteJetzt;

  for (int cnt=0; cnt<KesselzeitCnt; cnt++){
    int ZeitStartProgramm=Kesselzeit[cnt][1];
    int ZeitEndeProgramm=Kesselzeit[cnt][2];
    if(Kesselzeit[cnt][0]==TagJetzt){ // Tag aus Array 
      if(ZeitJetzt == ZeitStartProgramm){ // Stunde liegt zwischen
        Nacht=false;
      } else if(ZeitJetzt == ZeitEndeProgramm){
        Nacht=true;
      }
    }
  }
  return Nacht;
}


// Kesselsteuerung
void Kesseltemperatur(bool Nacht) {
  bool Relais; // Zum Debuggen
  char buf[100];
  float KesselSollAnalog = analogRead(KesselPoti) +1;  // Wert darf nicht Null werden
  float Digits = (90/1024);
 // float KesselSoll = Digits * KesselSollAnalog; // Sollwert der Kesseltemperatur (max. 90°C) errechnen (+1 wg. *0!!)
  float KesselSoll = (90 *  KesselSollAnalog) / 1014;
  if (Nacht == true) {
    KesselSoll = KesselSoll - Absenkung; // Nachtabsenkung
  }

  float KesselTemperatur = getTemp(SensorKesseltemp); // Aktuelle Kesseltemp. holen ----> Adresse von Fühler!

  if (KesselTemperatur >= KesselSoll) { // Brenner ausschalten
    digitalWrite(BrennerRelais, LOW);
    Relais = LOW;
  }
  else if (KesselTemperatur <= (KesselSoll - HysterreseTemp)) { // Brenner einschalten
    digitalWrite(BrennerRelais, HIGH);
    Relais = HIGH;
  }
    Serial.print("Kesseltemp: ");
    Serial.println(KesselTemperatur);
    Serial.print("Kesselsolltemp: ");
    Serial.println(KesselSoll);
}


// TODO: Mehrere Fühler auswählbar machen.
float getTemp(int FuehlerNummer) {
   // Aufruf der Funktion sensors.requestTemperatures()
   // Dadurch werden alle werte abgefragt.
   Serial.print("Abfrage der Temperatur... ");
   sensors.requestTemperatures();
   Serial.println("DONE");
    float Temperatur = sensors.getTempCByIndex(FuehlerNummer);

   
   // Ausgabe der Daten für jeden Sensor
   for(int i=0 ;i<numberOfDevices; i++) {
      float tempC = sensors.getTempCByIndex(i);
      Serial.print("Sensor ");
      Serial.print(i, DEC);
      Serial.print(" hat grad Celsius: ");
      Serial.println(tempC);
   }
   return Temperatur;
}

